#r "Newtonsoft.Json"
#load "slackclient.csx"
#load "payload.csx"
using System;
using System.Net;
using System.Text;
using Newtonsoft.Json;
using KenticoCloud.Delivery;
using System.Configuration;
using System.Security.Cryptography;

private static bool blnValid = false;
private static bool blnPublish = false;
private static string strSlackURL = "[Slack Webhook URL]";
private static string strSlackUsername = "[Slack User]";
private static string strSlackChannel = "[Slack channel]";
private static string strWebhookSecret = "[Kentico Cloud Webhook Secret]";
private static string strKenticoCloudProjectID = "[Kentico Cloud Project ID]";
private static string strKenticoCloudPreviewAPIKey = "[Kentico Cloud Preview API Key]";

public static async Task<object> Run(HttpRequestMessage req, TraceWriter log)
{
    System.Net.ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;

    // Get the signature for validation
    IEnumerable<string> headerValues = req.Headers.GetValues("X-KC-Signature");
    var sig = headerValues.FirstOrDefault();

    // Get the content
    var content = req.Content;
    string jsonContent = content.ReadAsStringAsync().Result;

    // Generate a hash using the content and the webhook secret
    var hash = GenerateHash(jsonContent, strWebhookSecret);

    // Verify the notification is valid
    if(sig != hash)
    {
        log.Info("Unauthorized attempt!");
        log.Info(sig);
        log.Info(hash);
        return req.CreateResponse(HttpStatusCode.Unauthorized, new
        {
            error = "Unauthorized!"
        });
    }
    
    var settings = new JsonSerializerSettings
                    {
                        NullValueHandling = NullValueHandling.Ignore,
                        MissingMemberHandling = MissingMemberHandling.Ignore
                    };

    dynamic data = JsonConvert.DeserializeObject(jsonContent, settings);

    if (data == null)
    {
        return req.CreateResponse(HttpStatusCode.BadRequest, new
        {
            error = "Please pass data properties in the input object"
        });
    }

    // Determine the operation
    // Only process if it is publish or unpublish
    string strOperation = data.message.operation.ToString().ToLower();
    switch(strOperation)
    {
        case "publish":
            blnValid = true;
            blnPublish = true;
            break;
        case "unpublish":
            blnValid = true;
            blnPublish = false;
            break;
    }

    // Make sure it's a valid operation
    if(blnValid)
    {
        List<string> lstCodeNames = new List<string>();
  
        foreach(var item in data.data.items)
        {
            lstCodeNames.Add(item.codename.ToString());
        }

        // Post to Slack
        if(lstCodeNames.Count > 0)
        {
            await PostToSlack(lstCodeNames, log);
        }

        return req.CreateResponse(HttpStatusCode.OK, new
        {
            greeting = $"Success!"
        });
    }
    else
    {
        return req.CreateResponse(HttpStatusCode.NotImplemented, new
        {
            greeting = $"Not Supported!"
        });
    }
}



private static string GenerateHash(string message, string secret)
{
    secret = secret ?? "";
    var encoding = new System.Text.UTF8Encoding();
    byte[] keyByte = encoding.GetBytes(secret);
    byte[] messageBytes = encoding.GetBytes(message);
    using (var hmacsha256 = new HMACSHA256(keyByte))
    {
        byte[] hashmessage = hmacsha256.ComputeHash(messageBytes);
        return Convert.ToBase64String(hashmessage);
    }
}

private static async Task PostToSlack(List<string> list, TraceWriter log)
{
    try
    {

        SlackClient slackclient = new SlackClient(strSlackURL);
        DeliveryClient deliveryclient = new DeliveryClient(strKenticoCloudProjectID, strKenticoCloudPreviewAPIKey);
            
        foreach(string codename in list)
        {
                
             // Get the details from Kentico Cloud
            DeliveryItemResponse response = await deliveryclient.GetItemAsync(codename);
            if(response != null)
            {
                var item = response.Item;         
                log.Info(item.GetString("name"));

                slackclient.PostMessage(username: strSlackUsername,
                    text: "Content update:" + item.GetString("name") + (blnPublish ? " PUBLISHED!" : " UNPUBLISHED!"),
                    channel: strSlackChannel);
            }
            else
            {
             log.Info($"Item not found!");
           }
        }
    }
    catch (Exception ex)
    {
        log.Info(ex.Message.ToString()); 
    }
}